from setuptools import setup, find_packages

VERSION = '0.2.0'

setup(
    name='peaq-py',
    version=VERSION,
    url='https://github.com/peaqnetwork/peaq-py/',
    author='Jay Pan',
    author_email='pan@eotlabs.io',
    description='peaq python sdk',
    packages=find_packages(),
    install_requires=[],
)
